/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2010, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.as.logging;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 */
public enum Namespace {
    // must be first
    UNKNOWN(null),

    LOGGING_1_0("urn:jboss:domain:logging:1.0"),

    LOGGING_1_1("urn:jboss:domain:logging:1.1"),

    LOGGING_1_2("urn:jboss:domain:logging:1.2"),

    LOGGING_1_3("urn:jboss:domain:logging:1.3"),

    LOGGING_1_4("urn:jboss:domain:logging:1.4"),

    LOGGING_1_5("urn:jboss:domain:logging:1.5"),
    ;

    /**
     * The current namespace version.
     */
    public static final Namespace CURRENT = LOGGING_1_5;

    private final String name;

    Namespace(final String name) {
        this.name = name;
    }

    /**
     * Get the URI of this namespace.
     *
     * @return the URI
     */
    public String getUriString() {
        return name;
    }

    private static final Map<String, Namespace> MAP;
    private static final List<Namespace> READABLE;

    static {
        final Map<String, Namespace> map = new HashMap<String, Namespace>();
        final List<Namespace> readable = new ArrayList<Namespace>();
        for (Namespace namespace : values()) {
            final String name = namespace.getUriString();
            if (name != null) {
                map.put(name, namespace);
                readable.add(namespace);
            }
        }
        MAP = map;
        READABLE = Collections.unmodifiableList(readable);
    }

    public static List<Namespace> readable() {
        return READABLE;
    }

    public static Namespace forUri(String uri) {
        final Namespace element = MAP.get(uri);
        return element == null ? UNKNOWN : element;
    }
}
